TASKS_STRUCT_FILE_FILENAME = "struct.json"
TASKS_STRUCT_FILE_PROP_DEP_DIRS = "dependency_directories"
TASKS_STRUCT_FILE_PROP_EXECUTORS = "executors"
TASKS_STRUCT_FILE_PROP_EXECUTORS_NAMESPACE = "namespace"
TASKS_STRUCT_FILE_PROP_EXECUTORS_CLASS = "class"
TASKS_STRUCT_FILE_PROP_EXECUTORS_PARAMS = "parameters"

TASKS_STRUCT_FILE_BASIC_PROPERTY_KEYS = [
    TASKS_STRUCT_FILE_PROP_DEP_DIRS,
    TASKS_STRUCT_FILE_PROP_EXECUTORS,
]

TASKS_DEPS_FILE_FILENAME = "dependencies.json"
TASKS_DEPS_FILE_PROP_DEP_NAME = "name"
TASKS_DEPS_FILE_PROP_DEP_VERSION = "version"
