
ARG_SPEECH_TEXT_DATA = "speech_data"
ARG_SETTINGS_CONTAINER = "settings"

ARG_OUTS_MECHANISMS_CARRIERS = "outs_mechanisms"
ARG_OUTS_MECHANISMS_MECHANISM_DEFAULT = "outs_mechanism_default"

ARG_INS_MECHANISMS_CARRIERS = "ins_mechanisms"
ARG_INS_MECHANISMS_MECHANISM_DEFAULT = "ins_mechanism_default"
